#!/usr/bin/env python3
"""
System Hardening Tool
Advanced system security hardening and configuration tool.
Parameters:
    - mode: Hardening mode (audit, enforce)
    - profile: Security profile to apply
    - output: Output file for results
Requires: psutil, winreg (Windows), pwd, grp (Linux) packages
"""
import argparse
import logging
from datetime import datetime
import json
import os
import sys
import platform
import subprocess
import socket
import pwd
import grp
import stat
from pathlib import Path
import psutil
try:
    import winreg
except ImportError:
    winreg = None

class SystemHardening:
    def __init__(self, mode='audit', profile='standard'):
        self.mode = mode.lower()
        self.profile = profile.lower()
        self.os_type = platform.system().lower()
        self.results = {}
        self.setup_logging()
        self.load_security_profiles()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def load_security_profiles(self):
        """Load security profiles based on OS type"""
        self.profiles = {
            'standard': {
                'password_policy': {
                    'min_length': 12,
                    'complexity': True,
                    'history': 5,
                    'max_age': 90,
                    'min_age': 1
                },
                'account_policy': {
                    'max_attempts': 5,
                    'lockout_duration': 30,
                    'admin_rename': True,
                    'guest_disable': True
                },
                'network_security': {
                    'disable_smb1': True,
                    'disable_llmnr': True,
                    'disable_netbios': True,
                    'enable_firewall': True
                },
                'filesystem_security': {
                    'restrict_write': True,
                    'enable_audit': True,
                    'remove_unneeded': True
                },
                'service_security': {
                    'disable_unused': True,
                    'secure_registry': True,
                    'secure_shares': True
                }
            },
            'high': {
                'password_policy': {
                    'min_length': 16,
                    'complexity': True,
                    'history': 10,
                    'max_age': 60,
                    'min_age': 1
                },
                'account_policy': {
                    'max_attempts': 3,
                    'lockout_duration': 60,
                    'admin_rename': True,
                    'guest_disable': True
                },
                'network_security': {
                    'disable_smb1': True,
                    'disable_llmnr': True,
                    'disable_netbios': True,
                    'enable_firewall': True,
                    'strict_firewall': True
                },
                'filesystem_security': {
                    'restrict_write': True,
                    'enable_audit': True,
                    'remove_unneeded': True,
                    'encrypt_sensitive': True
                },
                'service_security': {
                    'disable_unused': True,
                    'secure_registry': True,
                    'secure_shares': True,
                    'restrict_admin': True
                }
            }
        }

    def check_windows_security(self):
        """Check Windows-specific security settings"""
        if not winreg:
            return None

        windows_security = {
            'password_policy': {},
            'account_policy': {},
            'network_security': {},
            'service_security': {}
        }

        try:
            # Check password policy
            policy_key = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE,
                r"SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System",
                0,
                winreg.KEY_READ
            )
            
            windows_security['password_policy'] = {
                'password_complexity': self._get_reg_value(
                    policy_key,
                    'PasswordComplexity'
                ),
                'password_length': self._get_reg_value(
                    policy_key,
                    'MinimumPasswordLength'
                )
            }

            # Check account policy
            windows_security['account_policy'] = {
                'guest_account': self._check_account_status('Guest'),
                'admin_account': self._check_account_status('Administrator')
            }

            # Check network security
            windows_security['network_security'] = {
                'smb1_enabled': self._check_smb1_status(),
                'firewall_enabled': self._check_firewall_status()
            }

            # Check service security
            windows_security['service_security'] = {
                'running_services': self._get_running_services()
            }

        except Exception as e:
            self.logger.error(f"Error checking Windows security: {str(e)}")

        return windows_security

    def _get_reg_value(self, key, value_name):
        """Helper function to get registry value"""
        try:
            value, _ = winreg.QueryValueEx(key, value_name)
            return value
        except:
            return None

    def _check_account_status(self, account_name):
        """Check Windows account status"""
        try:
            result = subprocess.run(
                ['net', 'user', account_name],
                capture_output=True,
                text=True
            )
            return 'Account active' in result.stdout
        except:
            return None

    def _check_smb1_status(self):
        """Check SMBv1 protocol status"""
        try:
            result = subprocess.run(
                ['powershell', 'Get-WindowsOptionalFeature -Online -FeatureName SMB1Protocol'],
                capture_output=True,
                text=True
            )
            return 'Enabled' in result.stdout
        except:
            return None

    def _check_firewall_status(self):
        """Check Windows Firewall status"""
        try:
            result = subprocess.run(
                ['netsh', 'advfirewall', 'show', 'allprofiles'],
                capture_output=True,
                text=True
            )
            return 'ON' in result.stdout
        except:
            return None

    def _get_running_services(self):
        """Get list of running Windows services"""
        try:
            result = subprocess.run(
                ['net', 'start'],
                capture_output=True,
                text=True
            )
            services = result.stdout.split('\n')[4:-3]  # Remove header and footer
            return [s.strip() for s in services if s.strip()]
        except:
            return []

    def check_linux_security(self):
        """Check Linux-specific security settings"""
        linux_security = {
            'user_security': {},
            'filesystem_security': {},
            'network_security': {},
            'system_security': {}
        }

        try:
            # Check user security
            linux_security['user_security'] = {
                'root_login_disabled': self._check_root_login(),
                'sudo_users': self._get_sudo_users(),
                'password_policy': self._check_password_policy()
            }

            # Check filesystem security
            linux_security['filesystem_security'] = {
                'sticky_bits': self._check_sticky_bits(),
                'world_writable': self._find_world_writable(),
                'suid_files': self._find_suid_files()
            }

            # Check network security
            linux_security['network_security'] = {
                'listening_ports': self._get_listening_ports(),
                'iptables_rules': self._get_iptables_rules()
            }

            # Check system security
            linux_security['system_security'] = {
                'selinux_status': self._check_selinux(),
                'running_services': self._get_systemd_services()
            }

        except Exception as e:
            self.logger.error(f"Error checking Linux security: {str(e)}")

        return linux_security

    def _check_root_login(self):
        """Check if root login is disabled"""
        try:
            with open('/etc/ssh/sshd_config', 'r') as f:
                return 'PermitRootLogin no' in f.read()
        except:
            return None

    def _get_sudo_users(self):
        """Get list of users with sudo privileges"""
        try:
            result = subprocess.run(
                ['getent', 'group', 'sudo'],
                capture_output=True,
                text=True
            )
            return result.stdout.strip().split(':')[-1].split(',')
        except:
            return []

    def _check_password_policy(self):
        """Check Linux password policy settings"""
        try:
            with open('/etc/login.defs', 'r') as f:
                content = f.read()
                return {
                    'pass_max_days': self._get_login_defs_value(
                        content,
                        'PASS_MAX_DAYS'
                    ),
                    'pass_min_days': self._get_login_defs_value(
                        content,
                        'PASS_MIN_DAYS'
                    ),
                    'pass_warn_age': self._get_login_defs_value(
                        content,
                        'PASS_WARN_AGE'
                    )
                }
        except:
            return {}

    def _get_login_defs_value(self, content, setting):
        """Helper function to get login.defs values"""
        for line in content.split('\n'):
            if line.startswith(setting):
                return line.split()[-1]
        return None

    def _check_sticky_bits(self):
        """Check sticky bit on world-writable directories"""
        try:
            result = subprocess.run(
                ['find', '/', '-type', 'd', '-perm', '-002'],
                capture_output=True,
                text=True
            )
            return result.stdout.strip().split('\n')
        except:
            return []

    def _find_world_writable(self):
        """Find world-writable files"""
        try:
            result = subprocess.run(
                ['find', '/', '-type', 'f', '-perm', '-002'],
                capture_output=True,
                text=True
            )
            return result.stdout.strip().split('\n')
        except:
            return []

    def _find_suid_files(self):
        """Find SUID/SGID files"""
        try:
            result = subprocess.run(
                ['find', '/', '-type', 'f', '\\( -perm -4000 -o -perm -2000 \\)'],
                capture_output=True,
                text=True
            )
            return result.stdout.strip().split('\n')
        except:
            return []

    def _get_listening_ports(self):
        """Get listening network ports"""
        try:
            result = subprocess.run(
                ['netstat', '-tuln'],
                capture_output=True,
                text=True
            )
            return result.stdout.strip().split('\n')[2:]  # Skip header
        except:
            return []

    def _get_iptables_rules(self):
        """Get iptables rules"""
        try:
            result = subprocess.run(
                ['iptables', '-L'],
                capture_output=True,
                text=True
            )
            return result.stdout.strip().split('\n')
        except:
            return []

    def _check_selinux(self):
        """Check SELinux status"""
        try:
            with open('/etc/selinux/config', 'r') as f:
                content = f.read()
                return 'SELINUX=enforcing' in content
        except:
            return None

    def _get_systemd_services(self):
        """Get running systemd services"""
        try:
            result = subprocess.run(
                ['systemctl', 'list-units', '--type=service'],
                capture_output=True,
                text=True
            )
            return result.stdout.strip().split('\n')[1:-7]  # Remove header/footer
        except:
            return []

    def apply_security_settings(self):
        """Apply security settings based on profile"""
        if self.mode != 'enforce':
            return

        try:
            profile_settings = self.profiles[self.profile]
            
            if self.os_type == 'windows':
                self._apply_windows_settings(profile_settings)
            else:
                self._apply_linux_settings(profile_settings)
                
        except Exception as e:
            self.logger.error(
                f"Error applying security settings: {str(e)}"
            )

    def _apply_windows_settings(self, settings):
        """Apply Windows security settings"""
        try:
            # Apply password policy
            if settings['password_policy']['complexity']:
                subprocess.run([
                    'net', 'accounts',
                    '/minpwlen:' + str(settings['password_policy']['min_length']),
                    '/maxpwage:' + str(settings['password_policy']['max_age']),
                    '/minpwage:' + str(settings['password_policy']['min_age']),
                    '/uniquepw:' + str(settings['password_policy']['history'])
                ])

            # Apply account policy
            if settings['account_policy']['guest_disable']:
                subprocess.run(['net', 'user', 'Guest', '/active:no'])

            # Apply network security
            if settings['network_security']['disable_smb1']:
                subprocess.run([
                    'powershell',
                    'Disable-WindowsOptionalFeature -Online -FeatureName SMB1Protocol'
                ])

            if settings['network_security']['enable_firewall']:
                subprocess.run([
                    'netsh', 'advfirewall', 'set', 'allprofiles', 'state', 'on'
                ])

        except Exception as e:
            self.logger.error(
                f"Error applying Windows settings: {str(e)}"
            )

    def _apply_linux_settings(self, settings):
        """Apply Linux security settings"""
        try:
            # Apply password policy
            with open('/etc/login.defs', 'r+') as f:
                content = f.read()
                content = self._update_login_defs(
                    content,
                    'PASS_MAX_DAYS',
                    settings['password_policy']['max_age']
                )
                content = self._update_login_defs(
                    content,
                    'PASS_MIN_DAYS',
                    settings['password_policy']['min_age']
                )
                f.seek(0)
                f.write(content)
                f.truncate()

            # Apply filesystem security
            if settings['filesystem_security']['restrict_write']:
                subprocess.run([
                    'chmod', 'go-w', '/etc/passwd',
                    '/etc/shadow', '/etc/group', '/etc/gshadow'
                ])

            # Apply network security
            if settings['network_security']['enable_firewall']:
                subprocess.run(['ufw', 'enable'])

        except Exception as e:
            self.logger.error(
                f"Error applying Linux settings: {str(e)}"
            )

    def _update_login_defs(self, content, setting, value):
        """Helper function to update login.defs settings"""
        lines = content.split('\n')
        for i, line in enumerate(lines):
            if line.startswith(setting):
                lines[i] = f"{setting}\t{value}"
        return '\n'.join(lines)

    def generate_report(self):
        """Generate security analysis report"""
        try:
            report = {
                'timestamp': datetime.now().isoformat(),
                'system': {
                    'os': platform.system(),
                    'version': platform.version(),
                    'architecture': platform.machine()
                },
                'mode': self.mode,
                'profile': self.profile,
                'results': self.results
            }

            # Add summary section
            summary = {
                'total_checks': 0,
                'passed_checks': 0,
                'failed_checks': 0,
                'warnings': 0
            }

            if self.os_type == 'windows':
                self.results = self.check_windows_security()
            else:
                self.results = self.check_linux_security()

            # Calculate statistics
            for category in self.results.values():
                for check in category.values():
                    summary['total_checks'] += 1
                    if check is True:
                        summary['passed_checks'] += 1
                    elif check is False:
                        summary['failed_checks'] += 1
                    elif check is None:
                        summary['warnings'] += 1

            report['summary'] = summary
            return report
        except Exception as e:
            self.logger.error(f"Error generating report: {str(e)}")
            return None

    def save_report(self, report, output_file):
        """Save security report to file"""
        try:
            with open(output_file, 'w') as f:
                json.dump(report, f, indent=4)
            self.logger.info(f"Report saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving report: {str(e)}")

    def print_summary(self, report):
        """Print security analysis summary"""
        self.logger.info("\nSystem Security Analysis Summary:")
        self.logger.info(f"Operating System: {report['system']['os']}")
        self.logger.info(f"Version: {report['system']['version']}")
        self.logger.info(f"Mode: {report['mode']}")
        self.logger.info(f"Profile: {report['profile']}")
        
        if 'summary' in report:
            summary = report['summary']
            self.logger.info("\nCheck Results:")
            self.logger.info(f"Total Checks: {summary['total_checks']}")
            self.logger.info(f"Passed: {summary['passed_checks']}")
            self.logger.info(f"Failed: {summary['failed_checks']}")
            self.logger.info(f"Warnings: {summary['warnings']}")

        # Print detailed results
        self.logger.info("\nDetailed Results:")
        for category, checks in report['results'].items():
            self.logger.info(f"\n{category.replace('_', ' ').title()}:")
            for check, result in checks.items():
                status = (
                    "✓" if result is True
                    else "✗" if result is False
                    else "?"
                )
                self.logger.info(
                    f"{status} {check.replace('_', ' ').title()}"
                )

    def start_hardening(self, output_file=None):
        """Start system hardening process"""
        self.logger.info("Starting system security hardening")
        self.logger.info(f"Mode: {self.mode}")
        self.logger.info(f"Profile: {self.profile}")
        
        try:
            # Check current security settings
            if self.os_type == 'windows':
                self.results = self.check_windows_security()
            else:
                self.results = self.check_linux_security()

            # Apply security settings if in enforce mode
            if self.mode == 'enforce':
                self.apply_security_settings()

            # Generate and save report
            report = self.generate_report()
            if report and output_file:
                self.save_report(report, output_file)
            if report:
                self.print_summary(report)
                
        except KeyboardInterrupt:
            self.logger.info("Hardening interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during hardening: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="System Security Hardening Tool"
    )
    parser.add_argument(
        "--mode",
        choices=['audit', 'enforce'],
        default='audit',
        help="Hardening mode"
    )
    parser.add_argument(
        "--profile",
        choices=['standard', 'high'],
        default='standard',
        help="Security profile to apply"
    )
    parser.add_argument(
        "--output",
        help="Output file for security report"
    )
    args = parser.parse_args()

    hardening = SystemHardening(args.mode, args.profile)
    hardening.start_hardening(args.output)
